package org.firstinspires.ftc.teamcode.subsystem;

import static org.firstinspires.ftc.teamcode.configs.RobotConstants.clawClose;
import static org.firstinspires.ftc.teamcode.configs.RobotConstants.clawOpen;

import com.qualcomm.robotcore.hardware.HardwareMap;
import com.qualcomm.robotcore.hardware.Servo;

import org.firstinspires.ftc.robotcore.external.Telemetry;
import org.firstinspires.ftc.teamcode.util.action.Actions;
import org.firstinspires.ftc.teamcode.util.action.RunAction;

public class ClawSubsystem {

    public enum ClawState {
        CLOSED, OPEN
    }

    private Servo claw;
    private ClawState state;
    public RunAction openClaw, closeClaw;
    public Telemetry telemetry;

    public ClawSubsystem(HardwareMap hardwareMap, Telemetry telemetry) {
        this.claw = hardwareMap.get(Servo.class, "claw-servo");
        this.openClaw = new RunAction(this::openClaw);
        this.closeClaw = new RunAction(this::closeClaw);
        this.telemetry = telemetry;
    }

    public void setState(ClawState clawState) {
        if (clawState == ClawState.CLOSED) {
            claw.setPosition(clawClose);
        } else if (clawState == ClawState.OPEN) {
            claw.setPosition(clawOpen);
        }
        this.state = clawState;
    }

    public ClawState getState() {
        return this.state;
    }

    public void switchState() {
        if (state == ClawState.CLOSED) {
            setState(ClawState.OPEN);
        } else if (state == ClawState.OPEN) {
            setState(ClawState.CLOSED);
        }
    }

    public void openClaw() {
        setState(ClawState.OPEN);
    }

    public void closeClaw() {
        setState(ClawState.CLOSED);
    }

    public void init() {
        Actions.runBlocking(closeClaw);
    }

    public void start() {
        Actions.runBlocking(closeClaw);
    }

    public double getPosition() {
        return this.claw.getPosition();
    }

}