package org.firstinspires.ftc.teamcode.subsystem;

import static org.firstinspires.ftc.teamcode.configs.RobotConstants.liftPower;
import static org.firstinspires.ftc.teamcode.configs.RobotConstants.liftToFloorPos;
import static org.firstinspires.ftc.teamcode.configs.RobotConstants.liftToHighBucketPos;
import static org.firstinspires.ftc.teamcode.configs.RobotConstants.liftToLowBucketPos;

import androidx.annotation.NonNull;

import com.acmerobotics.dashboard.telemetry.TelemetryPacket;
import com.acmerobotics.roadrunner.Action;
import com.acmerobotics.roadrunner.ftc.Actions;
import com.qualcomm.robotcore.hardware.DcMotor;
import com.qualcomm.robotcore.hardware.DcMotorSimple;
import com.qualcomm.robotcore.hardware.HardwareMap;

public class LiftActionsSubsystem {

    public DcMotor lift;

    public enum LiftState {
        FLOOR, LOW_BUCKET, HIGH_BUCKET, FLOAT, SUBMARINE
    }

    private LiftState liftState;

    public LiftActionsSubsystem(HardwareMap hardwareMap) {
        lift = hardwareMap.get(DcMotor.class, "lift-motor");
    }

    public class MoveToPosition implements Action {
        private int positionValue;
        private LiftState positionState;

        public MoveToPosition(int positionValue, LiftState positionState) {
            this.positionValue = positionValue;
            this.positionState = positionState;
        }

        @Override
        public boolean run(@NonNull TelemetryPacket telemetryPacket) {
            setState(positionState);
            lift.setTargetPosition(positionValue);
            lift.setMode(DcMotor.RunMode.RUN_TO_POSITION);
            telemetryPacket.put("Lift State", positionState);
            telemetryPacket.put("Lift Position", lift.getCurrentPosition());
            boolean result = lift.getTargetPosition() - 15 < lift.getCurrentPosition() &&
                    lift.getCurrentPosition() < lift.getTargetPosition() + 15;
            return !result;
        }
    }

    public Action toFloorPosition() {
        return new MoveToPosition(liftToFloorPos, LiftState.FLOOR);
    }

    public Action toLowBucketPosition() {
        return new MoveToPosition(liftToLowBucketPos, LiftState.LOW_BUCKET);
    }

    public Action toHighBucketPosition() {
        return new MoveToPosition(liftToHighBucketPos, LiftState.HIGH_BUCKET);
    }

    public void switchState() {
        if (this.liftState == LiftState.FLOOR) {
            Actions.runBlocking(toLowBucketPosition());
        } else if (this.liftState == LiftState.LOW_BUCKET) {
            Actions.runBlocking(toHighBucketPosition());
        } else if (this.liftState == LiftState.HIGH_BUCKET) {
            Actions.runBlocking(toFloorPosition());
        }
    }

    public void init() {
        lift.setMode(DcMotor.RunMode.STOP_AND_RESET_ENCODER);
        lift.setDirection(DcMotorSimple.Direction.REVERSE);
        lift.setPower(liftPower);
    }

    private void setState(LiftState liftState) {
        this.liftState = liftState;
    }

    public LiftState getState() {
        return this.liftState;
    }

    public int getPosition() {
        return lift.getCurrentPosition();
    }

    public void setPosition(int position) {
        lift.setTargetPosition(position);
        lift.setMode(DcMotor.RunMode.RUN_TO_POSITION);
    }

    public void start() {
        Actions.runBlocking(toFloorPosition());
    }

}
