package org.firstinspires.ftc.teamcode.subsystem;

import static org.firstinspires.ftc.teamcode.configs.RobotConstants.wristBucket;
import static org.firstinspires.ftc.teamcode.configs.RobotConstants.wristFloor;

import androidx.annotation.NonNull;

import com.acmerobotics.dashboard.telemetry.TelemetryPacket;
import com.acmerobotics.roadrunner.Action;
import com.acmerobotics.roadrunner.ftc.Actions;
import com.qualcomm.robotcore.hardware.HardwareMap;
import com.qualcomm.robotcore.hardware.ServoImplEx;

public class WristActionsSubsystem {

    public enum WristState {
        FLOOR, BUCKET
    }

    public ServoImplEx wrist;
    public WristState state;

    public WristActionsSubsystem(HardwareMap hardwareMap) {
        this.wrist = hardwareMap.get(ServoImplEx.class, "wrist-servo");
    }

    public class MoveToPosition implements Action {
        private double positionValue;
        private WristState positionState;

        public MoveToPosition(double positionValue, WristState positionState) {
            this.positionValue = positionValue;
            this.positionState = positionState;
        }

        @Override
        public boolean run(@NonNull TelemetryPacket telemetryPacket) {
            setState(positionState);
            wrist.setPosition(positionValue);
            telemetryPacket.put("Wrist State", positionState);
            return false;
        }
    }

    public Action toFloorPosition() {
        return new MoveToPosition(wristFloor, WristState.FLOOR);
    }

    public Action toBucketPosition() {
        return new MoveToPosition(wristBucket, WristState.BUCKET);
    }

    public void setState(WristState wristState) {
        this.state = wristState;
    }

    public void switchState() {
        if (state == WristState.FLOOR) {
            Actions.runBlocking(this.toBucketPosition());
        } else if (state == WristState.BUCKET) {
            Actions.runBlocking(this.toFloorPosition());
        }
    }

    public WristState getState() {
        return this.state;
    }

    public void init() {
        wrist.resetDeviceConfigurationForOpMode();
        Actions.runBlocking(this.toFloorPosition());
    }

    public void start() {
        Actions.runBlocking(this.toFloorPosition());
    }

    public void setPosition(double position) {
        wrist.setPosition(position);
    }

    public double getPosition() {
        return wrist.getPosition();
    }

}
